@@ -26,9 +26,9 @@
  * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#ifndef MAPFORMAT_H
-#define MAPFORMAT_H
+#pragma once
 
+#include "fileformat.h"
 #include "pluginmanager.h"
 
 #include <QObject>
@@ -39,61 +39,6 @@ namespace Tiled {
 
 class Map;
 
-class TILEDSHARED_EXPORT FileFormat : public QObject
-{
-    Q_OBJECT
-
-public:
-    enum Capability {
-        NoCapability    = 0x0,
-        Read            = 0x1,
-        Write           = 0x2,
-        ReadWrite       = Read | Write
-    };
-    Q_DECLARE_FLAGS(Capabilities, Capability)
-
-    explicit FileFormat(QObject *parent = nullptr)
-        : QObject(parent)
-    {}
-
-    /**
-     * Returns whether this format has Read and/or Write capabilities.
-     */
-    virtual Capabilities capabilities() const { return ReadWrite; }
-
-    /**
-     * Returns whether this format has all given capabilities.
-     */
-    bool hasCapabilities(Capabilities caps) const
-    { return (capabilities() & caps) == caps; }
-
-    /**
-     * Returns the absolute paths for the files that will be written by
-     * the map writer.
-     */
-    virtual QStringList outputFiles(const Map *, const QString &fileName) const
-    { return QStringList(fileName); }
-
-    /**
-     * Returns name filter for files in this map format.
-     */
-    virtual QString nameFilter() const = 0;
-
-    /**
-     * Returns whether this map format supports reading the given file.
-     *
-     * Generally would do a file extension check.
-     */
-    virtual bool supportsFile(const QString &fileName) const = 0;
-
-    /**
-     * Returns the error to be shown to the user if an error occured while
-     * trying to read a map.
-     */
-    virtual QString errorString() const = 0;
-};
-
-
 /**
  * An interface to be implemented for adding support for a map format to Tiled.
  * It can implement support for either loading or saving to a certain map
@@ -101,11 +46,24 @@ public:
  */
 class TILEDSHARED_EXPORT MapFormat : public FileFormat
 {
+    Q_OBJECT
+    Q_INTERFACES(Tiled::FileFormat)
+
 public:
     explicit MapFormat(QObject *parent = nullptr)
         : FileFormat(parent)
     {}
 
+    /**
+     * Returns the absolute paths for the files that will be written by
+     * this format for a given map.
+     *
+     * This is supported for Export formats only!
+     */
+    virtual QStringList outputFiles(const Map *, const QString &fileName) const
+    { return QStringList(fileName); }
+
+
     /**
      * Reads the map and returns a new Map instance, or 0 if reading failed.
      */
@@ -127,7 +85,6 @@ public:
 } // namespace Tiled
 
 Q_DECLARE_INTERFACE(Tiled::MapFormat, "org.mapeditor.MapFormat")
-Q_DECLARE_OPERATORS_FOR_FLAGS(Tiled::FileFormat::Capabilities)
 
 namespace Tiled {
 
@@ -140,6 +97,9 @@ class TILEDSHARED_EXPORT ReadableMapFormat : public MapFormat
     Q_INTERFACES(Tiled::MapFormat)
 
 public:
+    explicit ReadableMapFormat(QObject *parent = nullptr)
+        : MapFormat(parent)
+    {}
     Capabilities capabilities() const override { return Read; }
     bool write(const Map *, const QString &) override { return false; }
 };
@@ -154,6 +114,10 @@ class TILEDSHARED_EXPORT WritableMapFormat : public MapFormat
     Q_INTERFACES(Tiled::MapFormat)
 
 public:
+    explicit WritableMapFormat(QObject *parent = nullptr)
+        : MapFormat(parent)
+    {}
+
     Capabilities capabilities() const override { return Write; }
     Map *read(const QString &) override { return nullptr; }
     bool supportsFile(const QString &) const override { return false; }
@@ -161,44 +125,15 @@ public:
 
 
 /**
- * Convenience class that can be used when implementing file dialogs.
+ * Attempt to read the given map using any of the map formats added
+ * to the plugin manager, falling back to the TMX format if none are capable.
  */
-template<typename Format>
-class FormatHelper
-{
-public:
-    FormatHelper(FileFormat::Capabilities capabilities,
-                 QString initialFilter)
-        : mFilter(std::move(initialFilter))
-    {
-        PluginManager::each<Format>([this,capabilities](Format *format) {
-            if (format->hasCapabilities(capabilities)) {
-                const QString nameFilter = format->nameFilter();
+TILEDSHARED_EXPORT Map *readMap(const QString &fileName,
+                                QString *error = nullptr);
 
-                mFilter += QLatin1String(";;");
-                mFilter += nameFilter;
-
-                mFormats.append(format);
-                mFormatByNameFilter.insert(nameFilter, format);
-            }
-        });
-    }
-
-    const QString &filter() const
-    { return mFilter; }
-
-    const QList<Format*> &formats() const
-    { return mFormats; }
-
-    Format *formatByNameFilter(const QString &nameFilter) const
-    { return mFormatByNameFilter.value(nameFilter); }
-
-private:
-    QString mFilter;
-    QList<Format*> mFormats;
-    QMap<QString, Format*> mFormatByNameFilter;
-};
+/**
+ * Attempts to find a map format supporting the given file.
+ */
+TILEDSHARED_EXPORT MapFormat *findSupportingMapFormat(const QString &fileName);
 
 } // namespace Tiled
-
-#endif // MAPFORMAT_H
